
Functions
~~~~~~~~~

.. autofunction:: get_api_name

.. autofunction:: get_api_display_name

.. autofunction:: get_compiled_api

.. autofunction:: get_compiled_api_by_name

.. autofunction:: get_rtmidi_version


Exceptions
~~~~~~~~~~

.. autoclass:: rtmidi.RtMidiError
   :members:

.. autoclass:: rtmidi.InvalidPortError
   :members:

.. autoclass:: rtmidi.InvalidUseError
   :members:

.. autoclass:: rtmidi.MemoryAllocationError
   :members:

.. autoclass:: rtmidi.NoDevicesError
   :members:

.. autoclass:: rtmidi.SystemError
   :members:

.. autoclass:: rtmidi.UnsupportedOperationError
   :members:


Classes
~~~~~~~


.. autoclass:: rtmidi.MidiIn
   :members:
   :inherited-members:

.. autoclass:: rtmidi.MidiOut
   :members:
   :inherited-members:
